@@ -24,12 +24,13 @@ import (
 	"time"
 
 	"github.com/pborman/getopt/v2"    // BSD-3-Clause
+	"golang.org/x/net/idna"           // BSD-3-Clause
 	"golang.org/x/net/websocket"      // BSD-3-Clause
 	"salsa.debian.org/mdosch/xmppsrv" // BSD-2-Clause
 )
 
 const (
-	version             = "0.4.5"
+	version             = "0.5.1"
 	nsBOSH              = "urn:xmpp:alt-connections:xbosh"
 	nsC2SdTLS           = "urn:xmpp:alt-connections:tls"
 	nsC2SQuic           = "urn:xmpp:alt-connections:quic"
@@ -146,6 +147,11 @@ func main() {
 	case count > 1:
 		log.Fatal("Please specify only one server.")
 	}
+	// Convert server domain to ASCII.
+	serverASCII, err := idna.ToASCII(server[0])
+	if err != nil {
+		log.Fatal(err)
+	}
 
 	// Configure DNS resolver
 	c := xmppsrv.Config{
@@ -158,7 +164,7 @@ func main() {
 
 	// Set TLS config
 	var tlsConfig tls.Config
-	tlsConfig.ServerName = server[0]
+	tlsConfig.ServerName = serverASCII
 	tlsConfig.InsecureSkipVerify = false
 	switch *flagTLSVersion {
 	case 10:
@@ -181,7 +187,7 @@ func main() {
 	}
 
 	if *flagClient {
-		clientRecords, err := c.LookupClient(server[0])
+		clientRecords, err := c.LookupClient(serverASCII)
 		if err != nil && len(clientRecords) == 0 {
 			fmt.Println(err)
 			if *flagFallback && *flagTest {
@@ -197,14 +203,14 @@ func main() {
 			}
 		}
 		checkRecord(clientRecords, *flagVerbose, *flagV4, *flagV6, *flagTest,
-			&tlsConfig, timeout)
+			server[0], &tlsConfig, timeout)
 	}
 
 	if *flagServer {
 		if *flagClient {
 			fmt.Println()
 		}
-		serverRecords, err := c.LookupServer(server[0])
+		serverRecords, err := c.LookupServer(serverASCII)
 		if err != nil && len(serverRecords) == 0 {
 			fmt.Println(err)
 			if *flagFallback && *flagTest {
@@ -213,24 +219,24 @@ func main() {
 				serverRecords = []xmppsrv.SRV{
 					{
 						Type:   "xmpp-server",
-						Target: server[0],
+						Target: serverASCII,
 						Port:   5269,
 					},
 				}
 			}
 		}
 		checkRecord(serverRecords, *flagVerbose, *flagV4, *flagV6, *flagTest,
-			&tlsConfig, timeout)
+			server[0], &tlsConfig, timeout)
 	}
 
 	checkHostmeta(server[0], *flagVerbose, *flagV4, *flagV6,
-		*flagClient, *flagServer, *flagTest, &tlsConfig, timeout, c)
+		*flagClient, *flagServer, *flagTest, server[0], &tlsConfig, timeout, c)
 	checkHostmeta2(server[0], *flagVerbose, *flagV4, *flagV6,
-		*flagClient, *flagServer, *flagTest, &tlsConfig, timeout, c)
+		*flagClient, *flagServer, *flagTest, server[0], &tlsConfig, timeout, c)
 }
 
 func checkRecord(records []xmppsrv.SRV, verbose bool, ipv4 bool, ipv6 bool, test bool,
-	tlsConfig *tls.Config, timeout time.Duration,
+	serverName string, tlsConfig *tls.Config, timeout time.Duration,
 ) {
 	for count, record := range records {
 		if count > 0 {
@@ -238,7 +244,7 @@ func checkRecord(records []xmppsrv.SRV, verbose bool, ipv4 bool, ipv6 bool, test
 		}
 		printRecord(record)
 		if verbose && record.Target != "." {
-			printIP(record, ipv4, ipv6, test, tlsConfig, timeout)
+			printIP(record, ipv4, ipv6, test, serverName, tlsConfig, timeout)
 		}
 	}
 }
@@ -250,7 +256,7 @@ func printRecord(record xmppsrv.SRV) {
 }
 
 func printIP(record xmppsrv.SRV, ipv4 bool, ipv6 bool, test bool,
-	tlsConfig *tls.Config, timeout time.Duration,
+	serverName string, tlsConfig *tls.Config, timeout time.Duration,
 ) {
 	addresses, err := net.LookupIP(record.Target)
 	switch {
@@ -278,18 +284,18 @@ func printIP(record xmppsrv.SRV, ipv4 bool, ipv6 bool, test bool,
 				if err == nil {
 					switch record.Type {
 					case "xmpp-client":
-						startTLS("client", c, tlsConfig, timeout)
+						startTLS("client", serverName, c, tlsConfig, timeout)
 						c.Close()
 					case "xmpps-client":
 						tlsConfig.NextProtos = []string{"xmpp-client"}
-						directTLS("client", c, tlsConfig, timeout)
+						directTLS("client", serverName, c, tlsConfig, timeout)
 						c.Close()
 					case "xmpp-server":
-						startTLS("server", c, tlsConfig, timeout)
+						startTLS("server", serverName, c, tlsConfig, timeout)
 						c.Close()
 					case "xmpps-server":
 						tlsConfig.NextProtos = []string{"xmpp-server"}
-						directTLS("server", c, tlsConfig, timeout)
+						directTLS("server", serverName, c, tlsConfig, timeout)
 						c.Close()
 					default:
 						c.Close()
@@ -310,7 +316,7 @@ func connectionTest(server string, transport string, timeout time.Duration) (net
 	return c, err
 }
 
-func startTLS(recordType string, c net.Conn, tlsConfig *tls.Config, timeout time.Duration) {
+func startTLS(recordType string, serverName string, c net.Conn, tlsConfig *tls.Config, timeout time.Duration) {
 	// Created with https://github.com/miku/zek
 	type Proceed struct {
 		XMLName xml.Name `xml:"proceed"`
@@ -336,7 +342,7 @@ func startTLS(recordType string, c net.Conn, tlsConfig *tls.Config, timeout time
 	}
 	startStream := "<stream:stream xmlns:stream='http://etherx.jabber.org/streams' " +
 		"xmlns='jabber:" + recordType + "'" + from + " to='" +
-		tlsConfig.ServerName + "' version='1.0'>"
+		serverName + "' version='1.0'>"
 	_, err := c.Write([]byte(startStream))
 	if err != nil {
 		fmt.Println(statusNOK)
@@ -444,7 +450,7 @@ func startTLS(recordType string, c net.Conn, tlsConfig *tls.Config, timeout time
 	}
 }
 
-func directTLS(recordType string, conn net.Conn, tlsConfig *tls.Config, timeout time.Duration) {
+func directTLS(recordType string, serverName string, conn net.Conn, tlsConfig *tls.Config, timeout time.Duration) {
 	var serverStreamError StreamError
 	c := tls.Client(conn, tlsConfig)
 	ctx := context.Background()
@@ -473,7 +479,7 @@ func directTLS(recordType string, conn net.Conn, tlsConfig *tls.Config, timeout
 		}
 		startStream := "<stream:stream xmlns:stream='http://etherx.jabber.org/streams' " +
 			"xmlns='jabber:" + recordType + "'" + from + " to='" +
-			tlsConfig.ServerName + "' version='1.0'>"
+			serverName + "' version='1.0'>"
 		_, err := c.Write([]byte(startStream))
 		if err != nil {
 			fmt.Println(statusNOK)
@@ -524,8 +530,8 @@ func directTLS(recordType string, conn net.Conn, tlsConfig *tls.Config, timeout
 	}
 }
 
-func checkHostmeta(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool, s2s bool,
-	test bool, tlsConfig *tls.Config, timeout time.Duration, dnsConfig xmppsrv.Config,
+func checkHostmeta(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool, s2s bool, test bool,
+	serverName string, tlsConfig *tls.Config, timeout time.Duration, dnsConfig xmppsrv.Config,
 ) {
 	type hostmetaXML struct {
 		XMLName xml.Name `xml:"XRD"`
@@ -594,8 +600,8 @@ func checkHostmeta(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool,
 	}
 }
 
-func checkHostmeta2(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool, s2s bool,
-	test bool, tlsConfig *tls.Config, timeout time.Duration, dnsConfig xmppsrv.Config,
+func checkHostmeta2(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool, s2s bool, test bool,
+	serverName string, tlsConfig *tls.Config, timeout time.Duration, dnsConfig xmppsrv.Config,
 ) {
 	type hostmetaJSON struct {
 		XMPP struct {
@@ -705,7 +711,7 @@ func checkHostmeta2(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool,
 									continue
 								}
 								tlsConfig.NextProtos = []string{"xmpp-client"}
-								directTLS("client", c, tlsConfig, timeout)
+								directTLS("client", serverName, c, tlsConfig, timeout)
 								c.Close()
 							}
 						}
@@ -742,7 +748,7 @@ func checkHostmeta2(server string, verbose bool, ipv4 bool, ipv6 bool, c2s bool,
 									continue
 								}
 								tlsConfig.NextProtos = []string{"xmpp-server"}
-								directTLS("client", c, tlsConfig, timeout)
+								directTLS("client", serverName, c, tlsConfig, timeout)
 								c.Close()
 							}
 						}
@@ -861,6 +867,8 @@ func checkBOSH(server string, target string, ipv4 bool, ipv6 bool, tlsConfig *tl
 		fmt.Println(err)
 		return
 	}
+	tlsConfig.NextProtos = []string{"http/1.1", "h2"}
+	tlsConfig.ServerName = boshURI.Hostname()
 	httpClient, err := getHTTPClient(boshURI.Hostname(), boshURI.Port(), ipv4, ipv6, timeout, tlsConfig, srvConfig)
 	if err != nil {
 		fmt.Println(statusNOK)
@@ -937,9 +945,17 @@ func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout
 	}
 	wsTLSConfig := &tls.Config{
 		MinVersion:         tlsConfig.MinVersion,
-		ServerName:         server,
+		ServerName:         wsURI.Hostname(),
 		InsecureSkipVerify: false,
 	}
+	// Websocket connections currently fail when using HTTP/2, but also none of the
+	// alternative libraries support it, so currently it is disabled.
+	// TODO: Switch library, once one supports it.
+	// See:
+	// https://github.com/coder/websocket/issues/4
+	// https://github.com/gorilla/websocket/issues/417
+	// wsTLSConfig.NextProtos = []string{"http/1.1", "h2"}
+	wsTLSConfig.NextProtos = []string{"http/1.1"}
 	wsConfig := &websocket.Config{
 		Location:  wsURI,
 		Origin:    origURI,
